==> improv-0.0.1.tgz: improv-0.0.1/lib/thermocouple/thermocouple.c <==
/*
    thermocouple.c
    thermocouple library main source file

    libtc - a thermocouple library
    Copyright (C) 1998 Mark Rose <mrose@stm.lbl.gov>
    Copyright (C) 1999 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <unistd.h>
#include <math.h>
#include <thermocouple.h>

#include <type_b.h>
#include <type_e.h>
#include <type_j.h>
#include <type_k.h>
#include <type_n.h>
#include <type_r.h>
#include <type_s.h>
#include <type_t.h>

static struct thermocouple_struct tc_B={
	next: NULL,
	name: "B",
	emf_to_temp: TC_B_E2T,
	temp_to_emf: TC_B_T2E,
};

static struct thermocouple_struct tc_E={
	next: &tc_B,
	name: "E",
	emf_to_temp: TC_E_E2T,
	temp_to_emf: TC_E_T2E,
};

static struct thermocouple_struct tc_J={
	next: &tc_E,
	name: "J",
	emf_to_temp: TC_J_E2T,
	temp_to_emf: TC_J_T2E,
};

static struct thermocouple_struct tc_K={
	next: &tc_J,
	name: "K",
	emf_to_temp: TC_K_E2T,
	temp_to_emf: TC_K_T2E,
};

static struct thermocouple_struct tc_N={
	next: &tc_K,
	name: "N",
	emf_to_temp: TC_N_E2T,
	temp_to_emf: TC_N_T2E,
};

static struct thermocouple_struct tc_R={
	next: &tc_N,
	name: "R",
	emf_to_temp: TC_R_E2T,
	temp_to_emf: TC_R_T2E,
};

static struct thermocouple_struct tc_S={
	next: &tc_R,
	name: "S",
	emf_to_temp: TC_S_E2T,
	temp_to_emf: TC_S_T2E,
};

static struct thermocouple_struct tc_T={
	next: &tc_S,
	name: "T",
	emf_to_temp: TC_T_E2T,
	temp_to_emf: TC_T_T2E,
};

static thermocouple_t thermocouples=&tc_T;



#define Infinity	HUGE_VAL

#ifndef NAN
#define NAN \
  (__extension__ ((union { unsigned char __c[8]; double __d; })			\
		  {{ 0, 0, 0, 0, 0, 0, 0xf8, 0x7f }}).__d)
#endif



/* internal functions */

static double calculate_table(tc_table *pt,double x)
{
	double y=0;
	int n;

	for(n=pt->size-1;n>=0;n--){
		y=x*y+pt->d[n];
	}

	if(pt->a0 != 0){
		/* XXX fix this !!! */
		y += pt->a0 * exp( - (x*pt->a1 + pt->a2));
	}

	return y;
}

static tc_table *find_table(tc_table *t,double x)
{
	while(t){
		if(x>=t->min && x<=t->max)break;
		t=t->next;
	}
	return t;
}

static void generic_range(tc_table *t,double *min,double *max)
{
	double _min,_max;

	_min=t->min;
	_max=t->max;
	while(t){
		if(_min>t->min)_min=t->min;
		if(_max<t->max)_max=t->max;
		t=t->next;
	}
	if(min)*min=_min;
	if(max)*max=_max;
}



/* exported functions */

thermocouple_t tc_find_by_name(const char *name)
{
	thermocouple_t t=thermocouples;

	while(t){
		if(!strcmp(t->name,name))break;
		t=t->next;
	}
	return t;
}

double tc_temp_to_emf(thermocouple_t type,double temp)
{
	tc_table *table;

	table=find_table(type->temp_to_emf,temp);

	if(!table)
		return NAN;

	return calculate_table(table,temp);
}

double tc_emf_to_temp(thermocouple_t type,double emf)
{
	tc_table *table;

	table=find_table(type->emf_to_temp,emf);

	if(!table)
		return NAN;

	return calculate_table(table,emf);
}

void tc_emf_range(thermocouple_t type,double *min,double *max)
{
	generic_range(type->emf_to_temp,min,max);
}

void tc_temp_range(thermocouple_t type,double *min,double *max)
{
	generic_range(type->temp_to_emf,min,max);
}

char *tc_name(thermocouple_t type)
{
	return type->name;
}

thermocouple_t tc_browse(thermocouple_t t)
{
	if(!t)
		return thermocouples;

	return t->next;
}

==> improv-0.0.2.tgz: improv-0.0.2/lib/thermocouple/thermocouple.c <==
/*
    thermocouple.c
    thermocouple library main source file

    libtc - a thermocouple library
    Copyright (C) 1998 Mark Rose <mrose@stm.lbl.gov>
    Copyright (C) 1999 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <unistd.h>
#include <math.h>
#include <thermocouple.h>

#include <type_b.h>
#include <type_e.h>
#include <type_j.h>
#include <type_k.h>
#include <type_n.h>
#include <type_r.h>
#include <type_s.h>
#include <type_t.h>

static struct thermocouple_struct tc_B={
	next: NULL,
	name: "B",
	emf_to_temp: TC_B_E2T,
	temp_to_emf: TC_B_T2E,
};

static struct thermocouple_struct tc_E={
	next: &tc_B,
	name: "E",
	emf_to_temp: TC_E_E2T,
	temp_to_emf: TC_E_T2E,
};

static struct thermocouple_struct tc_J={
	next: &tc_E,
	name: "J",
	emf_to_temp: TC_J_E2T,
	temp_to_emf: TC_J_T2E,
};

static struct thermocouple_struct tc_K={
	next: &tc_J,
	name: "K",
	emf_to_temp: TC_K_E2T,
	temp_to_emf: TC_K_T2E,
};

static struct thermocouple_struct tc_N={
	next: &tc_K,
	name: "N",
	emf_to_temp: TC_N_E2T,
	temp_to_emf: TC_N_T2E,
};

static struct thermocouple_struct tc_R={
	next: &tc_N,
	name: "R",
	emf_to_temp: TC_R_E2T,
	temp_to_emf: TC_R_T2E,
};

static struct thermocouple_struct tc_S={
	next: &tc_R,
	name: "S",
	emf_to_temp: TC_S_E2T,
	temp_to_emf: TC_S_T2E,
};

static struct thermocouple_struct tc_T={
	next: &tc_S,
	name: "T",
	emf_to_temp: TC_T_E2T,
	temp_to_emf: TC_T_T2E,
};

static thermocouple_t thermocouples=&tc_T;



#define Infinity	HUGE_VAL

#ifndef NAN
#define NAN \
  (__extension__ ((union { unsigned char __c[8]; double __d; })			\
		  {{ 0, 0, 0, 0, 0, 0, 0xf8, 0x7f }}).__d)
#endif



/* internal functions */

static double calculate_table(tc_table *pt,double x)
{
	double y=0;
	int n;

	for(n=pt->size-1;n>=0;n--){
		y=x*y+pt->d[n];
	}

	if(pt->a0 != 0){
		y += pt->a0 * exp( pt->a1*(x - pt->a2)*(x - pt->a2));
	}

	return y;
}

static tc_table *find_table(tc_table *t,double x)
{
	while(t){
		if(x>=t->min && x<=t->max)break;
		t=t->next;
	}
	return t;
}

static void generic_range(tc_table *t,double *min,double *max)
{
	double _min,_max;

	_min=t->min;
	_max=t->max;
	while(t){
		if(_min>t->min)_min=t->min;
		if(_max<t->max)_max=t->max;
		t=t->next;
	}
	if(min)*min=_min;
	if(max)*max=_max;
}



/* exported functions */

thermocouple_t tc_find_by_name(const char *name)
{
	thermocouple_t t=thermocouples;

	while(t){
		if(!strcmp(t->name,name))break;
		t=t->next;
	}
	return t;
}

double tc_temp_to_emf(thermocouple_t type,double temp)
{
	tc_table *table;

	table=find_table(type->temp_to_emf,temp);

	if(!table)
		return NAN;

	return calculate_table(table,temp);
}

double tc_emf_to_temp(thermocouple_t type,double emf)
{
	tc_table *table;

	table=find_table(type->emf_to_temp,emf);

	if(!table)
		return NAN;

	return calculate_table(table,emf);
}

void tc_emf_range(thermocouple_t type,double *min,double *max)
{
	generic_range(type->emf_to_temp,min,max);
}

void tc_temp_range(thermocouple_t type,double *min,double *max)
{
	generic_range(type->temp_to_emf,min,max);
}

char *tc_name(thermocouple_t type)
{
	return type->name;
}

thermocouple_t tc_browse(thermocouple_t t)
{
	if(!t)
		return thermocouples;

	return t->next;
}
